<?php
	require_once(CI_PATH_LIBS.'/config.inc.php');
	require_once(CI_PATH_LIBS.'/auth.inc.php');
	require_once(CI_PATH_LIBS.'/users.inc.php');
	require_once(CI_PATH_LIBS.'/groups.inc.php');
	require_once(CI_PATH_LIBS.'/settings.inc.php');
	
	class CI_ADDRESSBOOK
	{
		private $level;
		
		function CI_ADDRESSBOOK($level='public')
		{
			$levels=$this->get_folders();
			if (!$levels[$level]) $level='public';
			
			$this->level=$level;
		}

		function get_config_path()
		{
			return IGX_PATH_BASE.'/config/maps';
		}

		function get_config_file($folder)
		{
			list($level, $arg)=split(':', $folder);

			if ($level == 'user') return $this->get_config_path().'/user-'.$arg.'.xml';
			if ($level == 'group') return $this->get_config_path().'/group-'.$arg.'.xml';

			return $this->get_config_path().'/public.xml';
		}

		function get_map($info)
		{
			return array(
				'shortcut'				=> $info['shortcut'],
				'destination'			=> $info['destination'],
				'caller_id'				=> $info['caller_id'],
				'caller_name'			=> $info['caller_name'],
				'ud_castiron_name'		=> $info['name'],
				'ud_castiron_surname'	=> $info['surname'],
				'ud_castiron_company'	=> $info['company'],
				'ud_castiron_email'		=> $info['email'],
				'ud_castiron_mobile'	=> $info['mobile'],
				'ud_castiron_office'	=> $info['office'],
				'ud_castiron_extension'	=> $info['extension'],
				'ud_castiron_home'		=> $info['home'],
				'ud_castiron_url'		=> $info['url']
			);
		}

		function get_rmap($info)
		{
			list($folder, $short) = split('/', $info['id']);
			list($level, $arg)=split(':', $folder);

			$user_id=NULL; $group_id=NULL;
			if ($level == 'user')		$user_id=$arg;
			elseif ($level == 'group')	$group_id=$arg;
			
			return array(
				'destination'	=> $info['destination'],
				'name'			=> $info['ud_castiron_name'],
				'caller_id'		=> $info['caller_id'],
				'caller_name'	=> $info['caller_name'],
				'surname'		=> $info['ud_castiron_surname'],
				'company'		=> $info['ud_castiron_company'],
				'email'			=> $info['ud_castiron_email'],
				'mobile'		=> $info['ud_castiron_mobile'],
				'office'		=> $info['ud_castiron_office'],
				'extension'		=> $info['ud_castiron_extension'],
				'home'			=> $info['ud_castiron_home'],
				'url'			=> $info['ud_castiron_url'],

				'shortcut'		=> $info['shortcut'],
				'user_id'		=> $user_id,
				'group_id'		=> $group_id
			);
		}

		function get_folder_name($level, $arg='')
		{
			if ($level == 'user') return 'user:'.$arg;
			if ($level == 'group') return 'group:'.$arg;
			return 'public';
		}

		function get_folders()
		{
			$out=array();
			$info=CI_AUTH::get_login_info();

			$manager_users	= new CI_USERS();
			$manager_groups	= new CI_GROUPS();

			$out['public']=ci_lbl('libaddressbook.level.public');

			if ($info['ud_castiron_role'] == 'admin')
			{
				$groups	= $manager_groups->get_list();
				$users	= $manager_users->get_list();

				foreach ($groups as $info2)
					$out['group:'.$info2['id']]=ci_lbl('libaddressbook.level.group', array('id' => $info2['id']));

				foreach ($users as $info2)
				{
					if (($info['id'] != 'admin') && ($info2['ud_castiron_role'] == 'admin') && ($info['id'] != $info2['id'])) continue;
					$out['user:'.$info2['id']]=ci_lbl('libaddressbook.level.user', array('id' => $info2['id']));
				}
			}
			
			elseif ($info['ud_castiron_role'] == 'groupadmin')
			{
				$users=$manager_users->get_list();

				$out['group:'.$info['group']]=ci_lbl('libaddressbook.level.mygroup');

				foreach ($users as $info2)
				{
					if ($info2['group'] != $info['group']) continue;
					if (($info2['ud_castiron_role'] == 'groupadmin') && ($info['id'] != $info2['id'])) continue;
					$out['user:'.$info2['id']]=ci_lbl('libaddressbook.level.user', array('id' => $info2['id']));
				}
			}

			else
			{
				$out['group:'.$info['group']]=ci_lbl('libaddressbook.level.mygroup');
				$out['user:'.$info['id']]=ci_lbl('libaddressbook.level.myuser');
			}

			return $out;
		}

		function get_list($where=NULL)
		{
			global $CI_DB;
			
			$list			= array();
			$list_public	= array();
			$list_group		= array();
			$list_user		= array();

			list($level, $arg)=split(':', $this->level);

			if ($where) $where="and ($where)";

			/* Read public addressbook */
			$list_public = $CI_DB->getAssoc("select shortcut as k, * from addressbook where user_id is null and group_id is null ".$where);

			/* Read group addressbook */
			if ($level == 'group')
			{
				$list_group = $CI_DB->getAssoc("select shortcut as k, * from addressbook where group_id=".$CI_DB->quoteSmart($arg).' '.$where);
				$group=$arg;
			}

			/* Read group + user addressbook */
			elseif ($level == 'user')
			{
				$manager=new CI_USERS();
				$info=$manager->get_info($arg);
				
				$list_group = $CI_DB->getAssoc("select shortcut as k, * from addressbook where group_id=".$CI_DB->quoteSmart($info['group']).' '.$where);
				$list_user 	= $CI_DB->getAssoc("select shortcut as k, * from addressbook where user_id=".$CI_DB->quoteSmart($arg).' '.$where);

				$user=$arg;
				$group=$info['group_id'];
			}

			/* merge the addressbooks */
			$list =& $list_public;

			if (is_array($list_group) && is_array($list_group))
				foreach ($list_group as $k => $v)
				{
					$list_group[$k]['folder']=$this->get_folder_name('group', $group);
					$list[$k] =& $list_group[$k];
				}

			if (is_array($list_user) && is_array($list_user))
				foreach ($list_user as $k => $v)
				{
					$list_user[$k]['folder']=$this->get_folder_name('user', $user);
					$list[$k] =& $list_user[$k];
				}

			/* Beautify... */
			if (!$list) $list=array();
			foreach ($list as $k => $v)
			{
				$folder=$list[$k]['folder'];

				$list[$k]=$this->get_map($list[$k]);
				$list[$k]['folder']=$folder;

				if (!$list[$k]['folder']) $list[$k]['folder']='public';
			}

			return $list;
		}

		function get_info($id)
		{
			global $CI_DB;
			
			$list =& $this->get_list("shortcut=".$CI_DB->quoteSmart($id));
			return $list[$id];
		}

		function shortcut_exists($s)
		{
			$list =& $this->get_list();
			if ($list[$s]) return true;

			return false;
		}

		function get_search($q)
		{
			global $CI_DB;

			$where=
				'shortcut = '.$CI_DB->quoteSmart($q).' or '.
				'caller_id = '.$CI_DB->quoteSmart($q).' or '.
				'caller_name = '.$CI_DB->quoteSmart($q).' or '.
				'destination like '.$CI_DB->quoteSmart("%$q%").' or '.
				'name like '.$CI_DB->quoteSmart("%$q%").' or '.
				'email like '.$CI_DB->quoteSmart("%$q%").' or '.
				'surname like '.$CI_DB->quoteSmart("%$q%").' or '.
				'company like '.$CI_DB->quoteSmart("%$q%")
			;

			$list =& $this->get_list($where);
			ksort($list);
			
			return $list;
		}

		function task_delete($id)
		{
			global $CI_DB;
			
			list($folder, $short)=split('/', $id);
			list($level, $arg)=split(':', $folder);

			switch ($level)
			{
				case 'public':
					$CI_DB->query("delete from addressbook where user_id is null and group_id is null and shortcut=".$CI_DB->quoteSmart($short));
					break;
					
				case 'group':
					$CI_DB->query("delete from addressbook where group_id=".$CI_DB->quoteSmart($arg)." and shortcut=".$CI_DB->quoteSmart($short));
					break;
					
				case 'user':
					$CI_DB->query("delete from addressbook where user_id=".$CI_DB->quoteSmart($arg)." and shortcut=".$CI_DB->quoteSmart($short));
					break;
			}
			
			return ci_response_ok();
		}
		
		function get_free_shortcut()
		{
			global $CI_DB;
			$res=$CI_DB->getOne("select MAX(shortcut)+1 from addressbook");
			
			return intval($res);
		}

		function task_save($info)
		{
			global $CI_DB;
			$CI_DB->query('BEGIN');
			
			if ($info['id'])
			{
				list($folder, $short) = split('/', $info['id']);
				list($level, $arg)=split(':', $folder);
			}
			else
			{
				$settings=new CI_SETTINGS_MISC();
				
				$folder	= 'public';
				$level	= '';
				$short	= $this->get_free_shortcut();
				
				$settings_info=$settings->get_info();
				
				$base=$settings_info['autoaddressbook_start'];
				
				$info['shortcut']=max($short, $base);
			}

			$fields =& $this->get_rmap($info);

			switch ($level)
			{
				case 'public':
					$where="user_id is null and group_id is null and shortcut=".$CI_DB->quoteSmart($short);
					break;
					
				case 'group':
					$where="group_id=".$CI_DB->quoteSmart($arg)." and shortcut=".$CI_DB->quoteSmart($short);
					break;
					
				case 'user':
					$where="user_id=".$CI_DB->quoteSmart($arg)." and shortcut=".$CI_DB->quoteSmart($short);
					break;
			}

			/* Check duplicates */
			if (($info['old_id'] != $info['id']) && $this->get_info($info['shortcut']))
			{
				$CI_DB->query('ROLLBACK');
				return ci_response_error(
					'unknown',
					array(
						'desc'		=> ci_lbl('libaddressbook.error.dusplicated-shortcut')
					)
				);
			}

			/* Remove the previous */
			if ($info['old_id'] && ($info['old_id'] != $info['id'])) $this->task_delete($info['old_id']);

			if ($this->get_info($info['shortcut']))
				$res=$CI_DB->autoExecute('addressbook', $fields, DB_AUTOQUERY_UPDATE, $where);
			else
				$res=$CI_DB->autoExecute('addressbook', $fields, DB_AUTOQUERY_INSERT);
			
			$CI_DB->query('COMMIT');

			if (!$info['id'])
				return ci_response_ok(array('shortcut' => $info['shortcut']));
			else
				return ci_response_ok();
		}
	}
?>