#!/usr/bin/env python

# Copyright and license: Please refer to LICENSE.TXT

import sys
import os
import os.path
import string
import re
import time
import sqlite3
from xml.sax import saxutils

_selfpath = (os.path.dirname(os.path.realpath(__file__)).split('/'))[:-1]
_selfpath.append('pylibs')
_libspath = '/'.join(_selfpath)
sys.path.append(_libspath)

from castiron import *

# Paths
IGX_PATH_BASE = APPS_PATH_BASE + '/igx'

# Check config parameters
CORE_CONF_SYSTEM		 = xml_to_array(CORE_PATH_BASE + '/config/system.xml')
CASTIRON_CONF_MISC		 = xml_to_array(CASTIRON_PATH_BASE + '/config/misc.xml')
IGX_CONF_SYSTEM			 = xml_to_array(IGX_PATH_BASE + '/config/system.xml')
CORE_CONF_SKYPE			 = xml_to_array(CORE_PATH_BASE + '/config/skype.xml', 0)

# Environmental data
from_skype = os.environ['from_skype']

# User lookup
S2P_USER_INFO = 0
for item in CORE_CONF_SKYPE['item'].values():
	if item['+']['skype'][0]['+']['username'][0]['#'] == from_skype:
		S2P_USER_INFO = xml_simplify_array(item)
		break

if not S2P_USER_INFO: sys.exit(-1)

###############################################################################################
# Check if account is master

do_exit = False

try:
	if S2P_USER_INFO['userdata']['igx-master-account'] == 'true':
		print '<?xml version="1.0" encoding="UTF-8"?>'
		print '<actions>'
		print '<action to="igx" name="set-var">'
		for k in os.environ: print '<arg name="' + k + '">' + s2p_xml_escape(os.environ[k]) + '</arg>'
		print '</action>'
		print '<action to="igx" name="get-free-user">'
		print '<arg name="file-route-in">routes/routes-in.xml</arg>'
		print '<on-error>'
		print '<action to="igx" name="script-xml">'
		print '<arg name="file">${_PATH_S2PAPPS_}/castiron/scripts/call-incoming-overflow_route-full.py</arg>'
		print '<arg name="async">false</arg>'
		print '</action>'
		print '</on-error>'
		print '</action>'
		print '<action to="igx" name="set-var">'
		print '<arg name="remote-user-name">${user-name}</arg>'
		print '<arg name="user-name">${free-user-name}</arg>'
		print '</action>'
		print '<action to="skype" name="call-transfer">'
		print '<on-error>'
		print '<action to="skype" name="call-finish"></action>'
		print '<action to="igx" name="exit"></action>'
		print '</on-error>'
		print '</action>'
		print '</actions>'

		do_exit = True

except:
	1

if do_exit: sys.exit(0)

IGX_DATABASE = sqlite3.connect(IGX_PATH_BASE + '/config/' + IGX_CONF_SYSTEM['database'], check_same_thread=False)
cur = IGX_DATABASE.cursor()

###############################################################################################
# Check for a new entry

if \
	CASTIRON_CONF_MISC['auto-addressbook']['enabled'] == 'true' and CASTIRON_CONF_MISC['auto-addressbook']['start'] and \
	os.environ['user_name'] and os.environ['user_name'][0] != '+' \
:
	cur.execute("select shortcut from addressbook where user_id is null and group_id is null and destination='%s'" % (os.environ['user_name']))
	res = cur.fetchone()

	if not res and os.environ['user_name'][0:17] != 'undisclosed_pstn:':
		cur.execute("select MAX(0+shortcut) from addressbook where user_id is null and group_id is null order by shortcut ")
		res = cur.fetchone()

		if res and res[0]:
			n = int(res[0]) + 1
		else:
			n = 1

		if n < int(CASTIRON_CONF_MISC['auto-addressbook']['start']): n = CASTIRON_CONF_MISC['auto-addressbook']['start']

		fields = {
			'destination'	: os.environ['user_name'],
			'shortcut'		: str(n)
		}

		try:
			[profile_name, profile_surname] = re.split(r'\s+', os.environ['user_dispname'], 1)
			fields['name'] = profile_name
			fields['surname'] = profile_surname
		except:
			fields['name'] = os.environ['user_dispname']

		fields_keys		 = ', '.join(fields.keys())
		fields_values_qm = []
		for f in fields.keys(): fields_values_qm.append('?')
		fields_values = ','.join(fields_values_qm)

		cur.execute("insert into addressbook(" + fields_keys + ") values (" + fields_values + ")", fields.values())
		IGX_DATABASE.commit()

###############################################################################################
# Get caller-id

fields_map = {
	'shortcut'		: 0,
	'caller_id'		: 1,
	'caller_name'	: 2,
	'email'			: 3,
	'name'			: 4,
	'surname'		: 5,
	'mobile'		: 6,
	'office'		: 7,
	'home'			: 8,
	'company'		: 9
}

cur.execute("select shortcut, caller_id, caller_name, email, name, surname, mobile, office, home, company from addressbook where user_id is null and group_id is null and destination='%s'" % (os.environ['user_name']))
res = cur.fetchone()

# Defaults no addressbook
if CASTIRON_CONF_MISC['caller-id']['caller-id'] == 'username': caller_id = os.environ['user_name']
elif CASTIRON_CONF_MISC['caller-id']['caller-id'] == 'userdispname': caller_id = os.environ['user_dispname']

if CASTIRON_CONF_MISC['caller-id']['caller-name'] == 'username': caller_name = os.environ['user_name']
elif CASTIRON_CONF_MISC['caller-id']['caller-name'] == 'userdispname': caller_name = os.environ['user_dispname']

shortcut=''

# Override
if res:
	shortcut = res[fields_map['shortcut']]

	# Defaults addressbook
	if fields_map.has_key(CASTIRON_CONF_MISC['caller-id']['caller-id']):
		caller_id	 = res[fields_map[CASTIRON_CONF_MISC['caller-id']['caller-id']]]

	if fields_map.has_key(CASTIRON_CONF_MISC['caller-id']['caller-name']):
		caller_name	 = res[fields_map[CASTIRON_CONF_MISC['caller-id']['caller-name']]]

	if res[fields_map['caller_id']]:	caller_id = res[fields_map['caller_id']]
	if res[fields_map['caller_name']]:	caller_name = res[fields_map['caller_name']]

# SkypeIN
elif len(os.environ['user_name']) and os.environ['user_name'][0] == '+':
	shortcut	 = re.sub('^\+', '00', os.environ['user_name'])
	caller_id	 = re.sub('^\+', '00', os.environ['user_name'])
	caller_name	 = "SkypeIN"

if not shortcut: shortcut = '0'
if not caller_id: caller_id = '0'
if not caller_name: caller_name = '0'

if S2P_USER_INFO['userdata'].has_key('castiron-caller-prefix') and S2P_USER_INFO['userdata']['castiron-caller-prefix']:
	caller_id = S2P_USER_INFO['userdata']['castiron-caller-prefix'] + caller_id

elif CASTIRON_CONF_MISC['caller-id'].has_key('caller-id') and CASTIRON_CONF_MISC['caller-id']['caller-prefix']:
	caller_id = CASTIRON_CONF_MISC['caller-id']['caller-prefix'] + caller_id

# Output
print '<?xml version="1.0" encoding="UTF-8"?>'
print '<actions>'
print '<action to="igx" name="set-var">'
print '<arg name="caller-shortcut">' + s2p_xml_escape(shortcut) + '</arg>'
print '<arg name="caller-id">' + s2p_xml_escape(caller_id) + '</arg>'
print '<arg name="caller-name">' + s2p_xml_escape(caller_name) + '</arg>'
print '</action>'

print '<action to="igx" name="check-incoming-filter">'
print '<arg name="file">filters/filters-in.xml</arg>'
print '<on-error>'
print '<action to="skype" name="call-hangup">'
print '</action>'
print '<action to="igx" name="exit"></action>'
print '</on-error>'
print '</action>'

print '<action to="igx" name="route-in">'
print '<arg name="file-route">routes/routes-in.xml</arg>'
print '</action>'
print '</actions>'

cur.close()
