#!/usr/bin/env python

# Copyright and license: Please refer to LICENSE.TXT

import sys
import os
import os.path
import string
import time
import datetime
import signal
import random
import sqlite3
from datetime import date
from datetime import timedelta
from threading import Lock

_selfpath=(os.path.dirname(os.path.realpath(__file__)).split('/'))[:-1]
_selfpath.append('pylibs')
_libspath='/'.join(_selfpath)
sys.path.append(_libspath)

from twister import *

sys.path.append(CORE_PATH_BASE+'/pylibs')
sys.path.append(CORE_PATH_BASE+'/sdk/python')

from s2p import *
from s2p_base import *
from s2p_logger import *

STOP_TWISTER	= False
accounts_map	= {}
groups_info		= {}

twist_lock	= Lock()

##########################################################################################
# Paths

TWISTER_PATH_CONFIG = TWISTER_PATH_BASE+'/config'

IGX_PATH_BASE = APPS_PATH_BASE+'/igx'
IGX_PATH_CONFIG = IGX_PATH_BASE+'/config'

IGX_CONF_SYSTEM 	= xml_to_array(IGX_PATH_CONFIG+'/system.xml')
TWISTER_CONF_SYSTEM = xml_to_array(TWISTER_PATH_CONFIG+'/system.xml')
TWISTER_CONF_GROUPS = xml_to_array(TWISTER_PATH_CONFIG+'/groups.xml', 0)

if not TWISTER_CONF_SYSTEM.has_key('enabled') or TWISTER_CONF_SYSTEM['enabled'] != 'true':
	sys.exit(0)

try:
	TWISTER_REGISTRY	= xml_to_array('/tmp/twister-registry.xml')
except:
	TWISTER_REGISTRY	= {}

S2P_SKYPES_INFO 	= {}

##########################################################################################
# S2PCORE logging

def twister_log(level, message):
	s2p_log("twister", level, message)

##########################################################################################
# Signal handler

def sighandler_quit(signum, frame):
	STOP_TWISTER=True
	
	twister_log("info", "Twister stopped due to signal "+str(signum))
	s2p_deregister_pid('twister')

	print "Wait while closing..."
	sys.exit(0)

##########################################################################################
# Skype accounts

def reload_info():
	global S2P_SKYPES_INFO
	
	while True:
		try:
			conf_skype=xml_to_array(CORE_PATH_CONFIG+'/skype.xml', 0)
	
			S2P_SKYPES_INFO = {}
			if conf_skype.has_key('item'):
				for item in conf_skype['item'].values():
					account_name=item['+']['skype'][0]['+']['username'][0]['#']
	
					S2P_SKYPES_INFO[account_name]=xml_simplify_array(item)
	
			break
	
		except:
			s2p_log("twister", "warn", "One error occurred while reading '"+CORE_PATH_CONFIG+"/skype.xml', trying again in one sec")
			time.sleep(1)

##########################################################################################
# IGX callog

if IGX_CONF_SYSTEM.has_key('database'):
	IGX_DATABASE=sqlite3.connect(IGX_PATH_CONFIG+'/'+IGX_CONF_SYSTEM['database'], check_same_thread=False)

else:
	twister_log("error", "Twister cannot start without IGX call-log on sqllite")
	sys.exit(-1)

##########################################################################################
# Setup

pid=s2p_register_pid('twister')
twister_log("info", "Twister started with pid "+str(pid))

signal.signal(signal.SIGINT, sighandler_quit)
signal.signal(signal.SIGQUIT, sighandler_quit)
signal.signal(signal.SIGTERM, sighandler_quit)

##########################################################################################
# Config handler

def update_skype_config():
	global S2P_SKYPES_INFO
	
	fp=open(CORE_PATH_CONFIG+'/skype.xml', 'w')
	
	fp.write('<?xml version="1.0" encoding="utf-8"?>\n')
	fp.write('<config>\n')
	
	for account in S2P_SKYPES_INFO.keys():
		
		fp.write('<item>\n')
		
		for k in S2P_SKYPES_INFO[account].keys():
			
			v=S2P_SKYPES_INFO[account][k]

			if type(v) == str or type(v) == unicode:
				fp.write('<'+k+'>'+s2p_xml_escape(v)+'</'+k+'>\n')
				
			elif not v:
				fp.write('<'+k+' />\n')
				
			elif type(v) == dict:
				fp.write('<'+k+'>\n')
				
				for k2 in v.keys():
					v2=v[k2]
					
					if type(v2) == str or type(v2) == unicode:
						fp.write('<'+k2+'>'+s2p_xml_escape(v2)+'</'+k2+'>\n')
						
					elif not v2:
						fp.write('<'+k2+' />\n')
				
				fp.write('</'+k+'>\n')
	
		fp.write('</item>\n')
	
	fp.write('</config>\n')
	
	fp.close()

def update_twister_registry():
	global TWISTER_REGISTRY
	
	out ='<?xml version="1.0" encoding="UTF-8"?>\n'
	out+='<registry>\n'
	
	for account in TWISTER_REGISTRY.keys():
		out+='\t<'+account+'>\n'
		
		for k in TWISTER_REGISTRY[account].keys():
			out+='\t\t<'+k+'>'
			out+=str(TWISTER_REGISTRY[account][k])
			out+='</'+k+'>\n'
		
		out+='\t</'+account+'>\n'
	
	out+='</registry>\n'
	
	fp=open('/tmp/twister-registry.xml', 'w')
	fp.write(out)
	fp.close()

##########################################################################################
# Search free account

def search_free_account_in_group(group_id):
	global S2P_SKYPES_INFO
	global TWISTER_CONF_GROUPS
	
	for g in TWISTER_CONF_GROUPS['group'].values():
		group=g['+']
		
		if group['id'][0]['#'] == group_id:
		
			available=[]
			for i in group['items'][0]['+']['item'].values():
				item	= i['+']
				account	= item['account'][0]['#']
				
				if \
					TWISTER_REGISTRY[account].has_key('status') and \
					TWISTER_REGISTRY[account]['status'] == 'failure' and \
					TWISTER_REGISTRY[account]['status-timestamp'] > time.time()-86400 \
				:
					continue
				
				if S2P_SKYPES_INFO[account]['enabled'] == 'true': continue
				if not accounts_map[account]['threshold']: available.append(account)
			
			if len(available):
				random.shuffle(available)
				return available[0]
			
			break

	return None

def search_free_account(account_to_replace):
	global S2P_SKYPES_INFO
	global TWISTER_CONF_GROUPS
	
	if not accounts_map.has_key(account_to_replace): return
	if not accounts_map[account_to_replace].has_key('group'): return
	
	group_id	= accounts_map[account_to_replace]['group']
	
	for g in TWISTER_CONF_GROUPS['group'].values():
		group=g['+']
		
		if group['id'][0]['#'] == group_id:
		
			available=[]
			for i in group['items'][0]['+']['item'].values():
				item	= i['+']
				account	= item['account'][0]['#']
				
				if \
					TWISTER_REGISTRY[account].has_key('status') and \
					TWISTER_REGISTRY[account]['status'] == 'failure' and \
					TWISTER_REGISTRY[account]['status-timestamp'] > time.time()-86400 \
				:
					continue
				
				if account == account_to_replace: continue
				
				if S2P_SKYPES_INFO[account]['enabled'] == 'true': continue
				if not accounts_map[account]['threshold']: available.append(account)
			
			if len(available):
				random.shuffle(available)
				return available[0]
			
			break

	return None

def twist_account(account):
	global S2P_SKYPES_INFO
	global TWISTER_REGISTRY
	
	if not accounts_map.has_key(account): return
	if not accounts_map[account].has_key('group'): return
	
	group_id	= accounts_map[account]['group']
	new_account	= search_free_account(account)
				
	if new_account:
		S2P_SKYPES_INFO[account]['enabled']		= 'false'
		S2P_SKYPES_INFO[new_account]['enabled']	= 'true'
		
		set_twister_registry(new_account, 'active', '')

		update_skype_config()
		
		twister_log("info", "Replacing "+account+" with "+new_account)
		
		os.system(CORE_PATH_BASE+'/bin/session.sh %s stop' % (account))
		os.system(CORE_PATH_BASE+'/bin/session.sh %s start' % (new_account))
		
	else:
		S2P_SKYPES_INFO[account]['enabled']		='false'
		update_skype_config()
		
		twister_log("warn", "Cannot replace account "+account+", shutting down")
		os.system(CORE_PATH_BASE+'/bin/session.sh %s stop' % (account))

def set_twister_registry(account, status, reason):
	global TWISTER_REGISTRY
	
	if not TWISTER_REGISTRY.has_key(account): TWISTER_REGISTRY[account]={}
	
	TWISTER_REGISTRY[account]['status']				= status
	TWISTER_REGISTRY[account]['status-reason']		= reason
	TWISTER_REGISTRY[account]['status-timestamp']	= time.time()
	
	update_twister_registry()

##########################################################################################
# Check stop-on condition

def on_skype_event(source, event_name, event_args):
	global TWISTER_CONF_SYSTEM
	
	twist_lock.acquire()
	
	try:
	
		if event_name == "call-outgoing-terminated" and event_args.has_key('pstn-status'):
			m=re.search(r'^(?P<code>\d+) ', event_args['pstn-status'])
			
			if m:
				code=m.group('code')
				
				if TWISTER_CONF_SYSTEM['stop-on'].has_key('_'+code) and TWISTER_CONF_SYSTEM['stop-on']['_'+code] == 'true':
					
					print "> Account "+source+": "+event_args['pstn-status']
					
					twister_log("warn", "Account "+source+" has a code "+event_args['pstn-status'])
					
					set_twister_registry(source, 'failure', event_args['pstn-status'])
					twist_account(source)
					
	except:
		twister_log("error", "One error occurred while receiving an event from Skype")
	
	twist_lock.release()
			
##########################################################################################
# Check stop-on condition

if TWISTER_CONF_SYSTEM.has_key('stop-on'):
	s2ps=S2P_SKYPE()
	s2ps.register_callback('raw-events', on_skype_event)
	s2ps.start()

##########################################################################################
# Run

tablename		= IGX_CONF_SYSTEM['callog']['db-table']

# Start monitoring
while not STOP_TWISTER:
	
	twist_lock.acquire()
	
	delta_day	= date.today()-timedelta(days=1)
	delta_week	= date.today()-timedelta(days=7)
	delta_month	= date.today()-timedelta(days=30)
	
	reload_info()

	# Check time spent
	for account in S2P_SKYPES_INFO.keys(): 
		cur = IGX_DATABASE.cursor()
		
		if not accounts_map.has_key(account): accounts_map[account]={}
		
		try:
			cur.execute("select SUM(duration) from "+tablename+" where skype_local='%s' and direction='out' and date>'%s'" % (account, delta_day))
			res = cur.fetchone()
			minutes_day=float(res[0])/60.0
		except:
			minutes_day=0
		
		try:
			cur.execute("select SUM(duration) from "+tablename+" where skype_local='%s' and direction='out' and date>'%s'" % (account, delta_week))
			res = cur.fetchone()
			minutes_week=float(res[0])/60.0
		except:
			minutes_week=0
			
		try:
			cur.execute("select SUM(duration) from "+tablename+" where skype_local='%s' and direction='out' and date>'%s'" % (account, delta_month))
			res = cur.fetchone()
			minutes_month=float(res[0])/60.0
		except:
			minutes_month=0
		
		accounts_map[account]['day-spent']		= int(minutes_day)
		accounts_map[account]['week-spent']		= int(minutes_week)
		accounts_map[account]['month-spent']	= int(minutes_month)
		
		is_new=False
		if not TWISTER_REGISTRY.has_key(account):
			is_new=True
			TWISTER_REGISTRY[account]={}
		
		TWISTER_REGISTRY[account]['day-spent']		= accounts_map[account]['day-spent']
		TWISTER_REGISTRY[account]['week-spent']		= accounts_map[account]['week-spent']
		TWISTER_REGISTRY[account]['month-spent']	= accounts_map[account]['month-spent']
		
		if is_new:
			set_twister_registry(account, 'active', '')
		
		cur.close()
		
	print "%30s %6s %6s %6s" % ("account", "day", "week", "month")
	print "-----------------------------------------------------------------------------------"

	# Check limits
	groups_info={}
	for g in TWISTER_CONF_GROUPS['group'].values():
		group=g['+']
		
		groups_info[group['id'][0]['#']]=g['+']
		
		for i in group['items'][0]['+']['item'].values():
			item	= i['+']
			account	= item['account'][0]['#']
			
			if not accounts_map.has_key(account): accounts_map[account]={}
			
			accounts_map[account]['day-limit']		= int(item['day-limit'][0]['#'])
			accounts_map[account]['week-limit']		= int(item['week-limit'][0]['#'])
			accounts_map[account]['month-limit']	= int(item['month-limit'][0]['#'])
			accounts_map[account]['threshold']		= False
			accounts_map[account]['group']			= group['id'][0]['#']
			
			try:
				if \
					(accounts_map[account]['day-limit'] and (accounts_map[account]['day-limit'] < accounts_map[account]['day-spent'])) or \
					(accounts_map[account]['week-limit'] and (accounts_map[account]['week-limit'] < accounts_map[account]['week-spent'])) or \
					(accounts_map[account]['month-limit'] and (accounts_map[account]['month-limit'] < accounts_map[account]['month-spent'])) \
				:
					accounts_map[account]['threshold']	= True
					print "%30s %6d %6d %6d OVER" % (account, accounts_map[account]['day-spent'], accounts_map[account]['week-spent'], accounts_map[account]['month-spent'])
				else:
					print "%30s %6d %6d %6d" % (account, accounts_map[account]['day-spent'], accounts_map[account]['week-spent'], accounts_map[account]['month-spent'])
				
			except:
				print "%30s %6d %6d %6d" % (account, accounts_map[account]['day-spent'], accounts_map[account]['week-spent'], accounts_map[account]['month-spent'])
	
	print "==================================================================================="
	
	# Start rotation
	for g in TWISTER_CONF_GROUPS['group'].values():
		group		= g['+']
		group_id	= group['id'][0]['#']
		
		for i in group['items'][0]['+']['item'].values():
			item	= i['+']
			account	= item['account'][0]['#']
			
			if S2P_SKYPES_INFO[account]['enabled'] != 'true': continue
			
			# Activate new accounts
			if accounts_map[account]['threshold']:
				set_twister_registry(account, 'stopped', 'threshold-exceeded')
				twist_account(account)
			
	# Check minimum number of accounts
	groups_count={}
	for account in accounts_map.keys():
		if not accounts_map[account].has_key('group'): continue
		
		if not groups_count.has_key(accounts_map[account]['group']): groups_count[accounts_map[account]['group']]=0
		if S2P_SKYPES_INFO[account]['enabled'] == 'false': continue
		
		groups_count[accounts_map[account]['group']]=groups_count[accounts_map[account]['group']]+1
		
	for group_id in groups_count.keys():
		
		delta=int(groups_info[group_id]['min'][0]['#'])-int(groups_count[group_id])

		if delta>0:
			twister_log("warn", "Group "+group_id+" has only "+str(int(groups_count[group_id]))+" accounts running.")
		
			for i in range(0, delta):
				new_account=search_free_account_in_group(group_id)
				
				if new_account:
					twister_log("info", "Starting '"+new_account+"' to full group "+group_id)
					
					S2P_SKYPES_INFO[new_account]['enabled']		='true'
					
					set_twister_registry(new_account, 'active', '')
					update_skype_config()
					
					os.system(CORE_PATH_BASE+'/bin/session.sh %s start' % (new_account))
					
				else:
					twister_log("warn", "Group "+group_id+" has not enough accounts available")
					break
				
	twist_lock.release()

	c=0
	while not STOP_TWISTER:
		c=c+1
		time.sleep(1)
		
		if c>300: break
		
