#!/usr/bin/env python

import sys
import time
import thread

core_path_base = '/s2p/core'

sys.path.append(core_path_base + '/pylibs')
sys.path.append(core_path_base + '/sdk/python')

from s2p import *
from s2p_base import *
from s2p_logger import *

# Global vars ##################################################################

s2ps = None
wav_file = '/s2p/apps/autocall/prova.wav'

# Caller class #################################################################

class S2P_Caller:
	def __init__(self, account):
		self.is_calling = False
		self.call_id = None
		self.account = account

	def get_is_calling(self):
		return self.is_calling

	def get_call_id(self):
		return self.call_id

	def acquire(self):
		self.is_calling = True
		
	def trace(self, msg):
		print self.account+': '+msg

	def call(self, dest):
		global s2ps

		self.is_calling = True
		self.call_id = None

		self.trace('Calling '+dest)
		s2ps.command(self.account, "call-start", {
			'user-name': dest
		})

	def on_call_started(self, event_args):
		if not self.call_id:
			self.trace('Call started')
			
		self.call_id = event_args['call-id']

	def on_call_terminated(self, event_args):
		global s2ps
		s2ps.command(self.account, "call-finish", {
			'call-id': self.call_id
		})
		self.trace('Call terminated')

		self.is_calling = False

	# Call has been answered
	def on_call_answered(self, event_args):
		global s2ps
		global wav_file
		
		self.trace('Call answered')
		s2ps.command(self.account, "call-set-input-file", {
			'call-id': self.call_id,
			'file-name': wav_file
		})

	# Audio file is finished
	def on_call_audio_finished(self, event_args):
		global s2ps
		if not self.is_calling: return

		self.trace('Audio finished')
		s2ps.command(self.account, "call-finish", {
			'call-id': self.call_id
		})

		self.is_calling = False

# Handle events
def on_skype_event(source, event_name, event_args):

	if not callers.has_key(source):
		print "- Unknown account " + source
		return

	if event_name == "call-outgoing-start":
		callers[source].on_call_started(event_args)

	elif event_name == "call-outgoing-answered":
		callers[source].on_call_answered(event_args)

	elif event_name == "call-outgoing-terminated":
		callers[source].on_call_terminated(event_args)

	elif event_name == "call-input-finished":
		callers[source].on_call_audio_finished(event_args)

# Vars #########################################################################

src_list = []
dst_list = []

callers = {}

# Skype handler ################################################################

s2ps = S2P_SKYPE() # Load Skype manager
s2ps.register_callback('raw-events', on_skype_event) # Attach events handler

s2ps.start() # Start events handling thread

# Read registered accounts #####################################################

res = s2ps.command('', "list-registered", {})

for item in res['args'].values():
	src_list.append(item)
	callers[item] = S2P_Caller(item)

# Read list file ###############################################################

fp = open('list.txt', 'r')
for line in fp:
	line = re.sub(r'[\n\r]', '', line)
	dst_list.append(line)

# Report #######################################################################

print "Active accounts : " + str(len(src_list))
print "Destinations    : " + str(len(dst_list))

# Start calling ################################################################

def do_call(account, destination):
	callers[account].call(destination)

pos = 0

while pos < len(dst_list):

	for account in src_list:
		if callers[account].get_is_calling(): continue

		destination = dst_list[pos]
		print "Calling " + destination

		callers[account].acquire()
		thread.start_new_thread(do_call, (account, destination))

		pos = pos + 1

	time.sleep(1)

all_done = False
while not all_done:
	all_done = True
	for account in src_list:
		if callers[account].get_is_calling(): all_done = False
	time.sleep(1)

# Close ########################################################################

s2ps.stop() # Stop events handling thread

sys.exit(0)
