#!/usr/bin/env python

# Copyright and license: Please refer to LICENSE.TXT

########################################################################################
# Extra files

execfile(CORE_PATH_SDK+'/python/s2p_base/channels_call.py')
execfile(CORE_PATH_SDK+'/python/s2p_base/channels_message.py')

########################################################################################
# Channels management

class S2P_CHANNELS(Thread):

	def __init__(self):
		Thread.__init__(self)
		
		self.sysconf=xml_to_array(CORE_PATH_CONFIG+'/system.xml')
		
		self._sock_chan=s2p_usock()
		self._sock_chan.connect(self.sysconf['path']['socks']+'/channels-manager.sock')
		self._sock_chan.start()

		self._callback_functions={}
		self._calls={}
		self._calls_garbage={}
		self._messages={}

		self._events_history={}			# Keeps all the channels events of a call until it is destroyed

		self._garbage_timeout=15
		self._stop=False

		self._clean_lock=threading.Lock()

	def run(self):
		while self._sock_chan.is_connected():
			if self._sock_chan.wait_for_incoming():
				while self._sock_chan.get_buflen():
					#thread.start_new_thread(self._on_message, (self._sock_chan.read(), ))
					self._on_message(self._sock_chan.read())

		self._sock_chan.close()

	def get_classname(self):
		return "S2P_CHANNELS"

	def stop(self):
		self._stop=True
		self._sock_chan.close()

		del self._sock_chan

	def _call_on_garbage(self, callid):
		self._calls_garbage[callid]=time.time()+self._garbage_timeout

	def _calls_garbage_clean(self):
		self._clean_lock.acquire()

		to_delete=[]
		
		for callid in self._calls_garbage.keys():
			if self._calls_garbage[callid] >= time.time(): continue

			if self._calls.has_key(callid):				del self._calls[callid]
			if self._events_history.has_key(callid):	del self._events_history[callid]

			to_delete.append(callid)

		for callid in to_delete: del self._calls_garbage[callid]

		self._clean_lock.release()

	def _sock_send(self, message):
		msg_code=random_string()
		out=self._sock_chan.expect("#"+msg_code+" "+message, r'^#'+msg_code+' ')
		return re.sub(r'^#[^\s]+\s+', '', out)

	def _parse_message(self, message):
		m=re.match(r'^(?P<module>[^\.]+)\.(?P<command>[^\)]+)\((?P<arguments>.*)\)$', message)

		if not m: return ""

		module=m.group('module')
		command=m.group('command')
		arguments_txt=string_escaped_to_octal(m.group('arguments'))

		arguments=re.findall(r'([\w+\-_]+):"([^"]*)"', arguments_txt)
		if not arguments: arguments=()

		arguments_hash={}
		for arg in arguments: arguments_hash[arg[0]]=unescape_string(arg[1])

		return { "module": module, "command": command, "args": arguments_hash }

	def _compose_command(self, module, command, args):

		args_a=[]
		for k in args.keys():
			args_a.append(k+':"'+escape_string(args[k])+'"')

		return self._sock_send(module+'.'+command+'('+','.join(args_a)+')')

	# DTMF event
	def _on_message_event_dtmf(self, parsed):
		callid=parsed['args']['call-id']
		dtmf=parsed['args']['code']

		# Junk events
		if not self._calls.has_key(callid): return

		# Invoke callback
		if self._callback_functions.has_key('dtmf'):
			self._callback_functions['dtmf'](self._calls[callid], dtmf)

	# Call event
	def _on_message_event_call(self, parsed):
		if not parsed['args'].has_key('call-id'): return
		
		callid=parsed['args']['call-id']
		destroy_call=False

		if not self._events_history.has_key(callid): self._events_history[callid]=[]
		self._events_history[callid].append(parsed)

		# Junk events
		if \
			not self._calls.has_key(callid) and \
			parsed['args']["event"] != 'call-incoming-start' and \
			parsed['args']["event"] != 'call-outgoing-start' \
		: return

		# New call
		if parsed['args']["event"] == 'call-incoming-start' or parsed['args']["event"] == 'call-outgoing-start':
			if not self._calls.has_key(callid):
				self._calls_garbage_clean()
				self._calls[callid]=S2P_CHANNELS_CALL(self, parsed['module'], callid)

		# Call closed
		elif parsed['args']["event"] == 'call-incoming-terminated' or parsed['args']["event"] == 'call-outgoing-terminated':
			destroy_call=True

		# Call progresss 
		elif parsed['args']["event"] == 'call-status-progress':
			self._calls[callid].update(parsed['args']["status"])

		# Invoke callback
		if self._callback_functions.has_key('call'):
			self._callback_functions['call'](self._calls[callid], parsed['args']["event"])

		# Destroy the call if needed
		#if destroy_call: del self._calls[callid]
		if destroy_call and self._calls.has_key(callid):			self._call_on_garbage(callid)

	# Message event
	def _on_message_event_message(self, parsed):
		messageid=parsed['args']['message-id']
		destroy_message=False

		# New message
		if not self._messages.has_key(messageid) and parsed['args']['event'] != 'chatmessage-incoming-received': return

		if parsed['args']['event'] == 'chatmessage-incoming-received':
			self._messages[messageid]=S2P_CHANNELS_MESSAGE(self, parsed['module'], messageid)
			self._messages[messageid].update(parsed['args'])

		destroy_message=True

		# Invoke callback
		if self._callback_functions.has_key('message'):
			self._callback_functions['message'](self._messages[messageid], parsed['args']['event'])

		# Destroy the call if needed
		#if destroy_message: del self._messages[messageid]

	# Event
	def _on_message_event(self, parsed):
		if parsed['args']["event"][0:9] == 'call-dtmf':
			self._on_message_event_dtmf(parsed)
			
		elif parsed['args']["event"][0:5] == 'call-':
			self._on_message_event_call(parsed)

		elif parsed['args']["event"][0:12] == 'chatmessage-':
			self._on_message_event_message(parsed)

		# Invoke wildcard callback
		if self._callback_functions.has_key('raw-events'):
			self._callback_functions['raw-events'](parsed['module'], parsed['args']["event"], parsed['args'])

	# Message
	def _on_message(self, message):
		parsed=self._parse_message(message)
		
		if parsed['command'] == "event":
			self._on_message_event(parsed)

########################################################################################
# Events history

	def get_events_history(self, call_id):
		if not self._events_history.has_key(call_id): return {}
		return self._events_history[call_id]

########################################################################################
# Callbacks

	def register_callback(self, key, callback_function):
		self._callback_functions[key]=callback_function

########################################################################################
# Misc functions

	def is_error(self, res):
		if not res: return True
		if res['command'] == "error": return True
		return False

########################################################################################
# Direct commands

	def command(self, module, command, args):
		return self._parse_message(self._compose_command(module, command, args))

########################################################################################
# Call commands

	def get_calls(self):
		return self._calls

	def get_call(self, call_id):
		self._calls_garbage_clean()
		
		if not self._calls.has_key(call_id): return {}
		return self._calls[call_id]

	def call(self, module, args):
		res=self.command(module, "call_start", args)
		if self.is_error(res): return (False, res)
		
		callid=res['args']['call-id']

		self._calls_garbage_clean()
		self._calls[callid]=S2P_CHANNELS_CALL(self, res['args']['module'], callid)

		return (self._calls[callid], res)

########################################################################################
# Message commands

	def get_messages(self):
		return self._messages

	def message(self, module, args):
		res=self.command(module, "message", args)
		if self.is_error(res): return (False, res)

		messageid=res['args']['message-id']

		self._messages[messageid]=S2P_CHANNELS_MESSAGE(self, module, messageid)

		return (self._messages[messageid], res)

		
			
		

